[Setting category="Colors" name="Color name in plugins dropdown"]
bool colorDropdown = true;

[Setting category="Colors" name="Outer color" color]
vec4 OuterColor = vec4(1.f, 0.f, 0.f, 0.9f);
[Setting category="Colors" name="Inner color" color]
vec4 InnerColor = vec4(0.f, 0.f, 1.f, 0.9f);

[Setting category="Colors" name="xdd"]
bool xddMode = false;

// xdd image
nvg::Texture@ xddTexture = nvg::LoadTexture("xdd.png");

string menuOptionColor(bool option) {
    if (option) {
        return "\\$afa";
    } else {
        return "\\$faa";
    }
}


void RenderMenu() {
    string menuColor;
    if (colorDropdown) {
        menuColor = Text::FormatOpenplanetColor(vec3(0.8, 0.8, 0.8) + 0.2 * InnerColor.xyz);
    } else {
        menuColor = "\\$z";
    }
    if (UI::BeginMenu(Text::FormatOpenplanetColor(OuterColor.xyz) + Icons::Circle + menuColor + " Cactus indicator")) {
        if (UI::MenuItem(menuOptionColor(show) + "Show when driving", "", show)) {
            show = !show;
        }
        if (UI::MenuItem(menuOptionColor(showEditor) + "Show in editor", "", showEditor)) {
            showEditor = !showEditor;
        }
        if (UI::MenuItem(menuOptionColor(showUIHidden) + "Show when UI hidden", "", showUIHidden)) {
            showUIHidden = !showUIHidden;
        }
        UI::Separator();
        if (UI::MenuItem(menuOptionColor(showDistance) + "Smaller dots with distance", "", showDistance)) {
            showDistance = !showDistance;
        }
        UI::Text("Dot size");
        UI::SameLine();
        UI::SetNextItemWidth(160);
        dotSize = UI::SliderInt("##Dot size", dotSize, 2, 64);

        if (UI::MenuItem(menuOptionColor(useGradientFill) + "Use gradient colors", "", useGradientFill)) {
            useGradientFill = !useGradientFill;
        }
        if (UI::MenuItem(menuOptionColor(lowDrawQuality) + "Single-color dots", "", lowDrawQuality)) {
            lowDrawQuality = !lowDrawQuality;
        }
        if (UI::IsItemHovered()) {
            UI::BeginTooltip();
            UI::Text('Might increase performance but probably not noticable');
            UI::EndTooltip();
        }
        if (UI::MenuItem(menuOptionColor(xddMode) + "use xdd", "", xddMode)) {
            xddMode = !xddMode;
        }
        if (UI::IsItemHovered()) {
            nvg::BeginPath();
            nvg::Rect(vec2(UI::GetMousePos().x, UI::GetMousePos().y), vec2(32, 32));
            nvg::FillPaint(nvg::TexturePattern(vec2(UI::GetMousePos().x, UI::GetMousePos().y), vec2(32, 32), 0.f, xddTexture, 1.f));
            nvg::Fill();
            nvg::ClosePath();
        }
        UI::EndMenu();
    }
}


void Render() {
    if (!(((inMap && show) || (inEditor && showEditor)) && (showUIHidden || UI::IsGameUIVisible()))) {return;}

    nvg::Reset();
#if TMNEXT
    CSmArenaClient@ playground = cast<CSmArenaClient>(GetApp().CurrentPlayground);
    CGameCtnChallenge@ rootMap = cast<CGameCtnChallenge>(GetApp().RootMap);

    if(rootMap is null ||
        (!showEditor && (
        playground is null || 
        playground.Arena is null))) {
        reset();
        return;
        }


#elif MP4
    CTrackManiaRace@ playground = cast<CTrackManiaRace>(GetApp().CurrentPlayground);
    CGameCtnChallenge@ rootMap = cast<CGameCtnChallenge>(GetApp().RootMap);

    if(rootMap is null ||
        (!showEditor && (
        playground is null))) {
        // playground.Arena is null
        reset();
        return;
        }
        

#endif

    for (uint i = 0; i < cacti.Length; i++ ) {
        CactusData@ cactus = cacti[i];
        renderCactus(cactus);
    }
}

bool renderCactus(CactusData@ cacti) {
    if (Camera::IsBehind(cacti.coords)) {return false;}

    vec3 pos = Camera::ToScreen(cacti.coords);
    if (pos.z >= 0) {return false;} // only happens when loading map

    float size;
    if (showDistance) {
        float distance = Math::Sqrt(-pos.z/32);
        size = (2*dotSize*(distance+3))/(distance*4);
    } else {
        size = dotSize;
    }
    if (xddMode) {
        nvg::BeginPath();
        nvg::Rect(vec2(pos.x-size*1.5, pos.y-size*1.5), vec2(size*3, size*3));
        nvg::FillPaint(nvg::TexturePattern(vec2(pos.x-size*1.5, pos.y-size*1.5), vec2(size*3, size*3), 0., xddTexture, OuterColor.w));
        nvg::Fill();
        nvg::ClosePath();
    } else {
        if (lowDrawQuality || !useGradientFill) {
            nvg::BeginPath();
            nvg::Circle(pos.xy, size);
            nvg::FillColor(OuterColor);
            nvg::Fill();
            if (!lowDrawQuality) {
                nvg::BeginPath();
                nvg::Circle(pos.xy, (size*0.6));
                nvg::FillColor(InnerColor);
                nvg::Fill();
            }
        } else {
            nvg::BeginPath();
            nvg::Circle(pos.xy, size);
            nvg::FillPaint(nvg::RadialGradient(pos.xy, size/8, size, InnerColor, OuterColor));
            nvg::Fill();
        }
    }
    return true;
}
